@@ -7,19 +7,19 @@ from django.utils.translation import gettext_lazy as _
 
 @deconstructible
 class ASCIIUsernameValidator(validators.RegexValidator):
-    regex = r'^[\w.@+-]+$'
+    regex = r"^[\w.@+-]+\Z"
     message = _(
-        'Enter a valid username. This value may contain only English letters, '
-        'numbers, and @/./+/-/_ characters.'
+        "Enter a valid username. This value may contain only English letters, "
+        "numbers, and @/./+/-/_ characters."
     )
     flags = re.ASCII
 
 
 @deconstructible
 class UnicodeUsernameValidator(validators.RegexValidator):
-    regex = r'^[\w.@+-]+$'
+    regex = r"^[\w.@+-]+\Z"
     message = _(
-        'Enter a valid username. This value may contain only letters, '
-        'numbers, and @/./+/-/_ characters.'
+        "Enter a valid username. This value may contain only letters, "
+        "numbers, and @/./+/-/_ characters."
     )
     flags = 0
