@@ -1,12 +1,11 @@
 import ipaddress
+import math
 import re
-import warnings
 from pathlib import Path
 from urllib.parse import urlsplit, urlunsplit
 
 from django.core.exceptions import ValidationError
 from django.utils.deconstruct import deconstructible
-from django.utils.deprecation import RemovedInDjango41Warning
 from django.utils.encoding import punycode
 from django.utils.ipv6 import is_valid_ipv6_address
 from django.utils.regex_helper import _lazy_re_compile
@@ -72,8 +71,8 @@ class URLValidator(RegexValidator):
 
     # IP patterns
     ipv4_re = (
-        r"(?:0|25[0-5]|2[0-4]\d|1\d?\d?|[1-9]\d?)"
-        r"(?:\.(?:0|25[0-5]|2[0-4]\d|1\d?\d?|[1-9]\d?)){3}"
+        r"(?:0|25[0-5]|2[0-4][0-9]|1[0-9]?[0-9]?|[1-9][0-9]?)"
+        r"(?:\.(?:0|25[0-5]|2[0-4][0-9]|1[0-9]?[0-9]?|[1-9][0-9]?)){3}"
     )
     ipv6_re = r"\[[0-9a-f:.]+\]"  # (simple regex, validated later)
 
@@ -97,7 +96,7 @@ class URLValidator(RegexValidator):
         r"^(?:[a-z0-9.+-]*)://"  # scheme is validated separately
         r"(?:[^\s:@/]+(?::[^\s:@/]*)?@)?"  # user:pass authentication
         r"(?:" + ipv4_re + "|" + ipv6_re + "|" + host_re + ")"
-        r"(?::\d{1,5})?"  # port
+        r"(?::[0-9]{1,5})?"  # port
         r"(?:[/?#][^\s]*)?"  # resource path
         r"\Z",
         re.IGNORECASE,
@@ -122,17 +121,16 @@ class URLValidator(RegexValidator):
             raise ValidationError(self.message, code=self.code, params={"value": value})
 
         # Then check full URL
+        try:
+            splitted_url = urlsplit(value)
+        except ValueError:
+            raise ValidationError(self.message, code=self.code, params={"value": value})
         try:
             super().__call__(value)
         except ValidationError as e:
             # Trivial case failed. Try for possible IDN domain
             if value:
-                try:
-                    scheme, netloc, path, query, fragment = urlsplit(value)
-                except ValueError:  # for example, "Invalid IPv6 URL"
-                    raise ValidationError(
-                        self.message, code=self.code, params={"value": value}
-                    )
+                scheme, netloc, path, query, fragment = splitted_url
                 try:
                     netloc = punycode(netloc)  # IDN -> ACE
                 except UnicodeError:  # invalid domain part
@@ -143,7 +141,7 @@ class URLValidator(RegexValidator):
                 raise
         else:
             # Now verify IPv6 in the netloc part
-            host_match = re.search(r"^\[(.+)\](?::\d{1,5})?$", urlsplit(value).netloc)
+            host_match = re.search(r"^\[(.+)\](?::[0-9]{1,5})?$", splitted_url.netloc)
             if host_match:
                 potential_ip = host_match[1]
                 try:
@@ -157,7 +155,7 @@ class URLValidator(RegexValidator):
         # section 3.1. It's defined to be 255 bytes or less, but this includes
         # one byte for the length of the name and one byte for the trailing dot
         # that's used to indicate absolute names in DNS.
-        if len(urlsplit(value).hostname) > 253:
+        if splitted_url.hostname is None or len(splitted_url.hostname) > 253:
             raise ValidationError(self.message, code=self.code, params={"value": value})
 
 
@@ -196,34 +194,7 @@ class EmailValidator:
     )
     domain_allowlist = ["localhost"]
 
-    @property
-    def domain_whitelist(self):
-        warnings.warn(
-            "The domain_whitelist attribute is deprecated in favor of "
-            "domain_allowlist.",
-            RemovedInDjango41Warning,
-            stacklevel=2,
-        )
-        return self.domain_allowlist
-
-    @domain_whitelist.setter
-    def domain_whitelist(self, allowlist):
-        warnings.warn(
-            "The domain_whitelist attribute is deprecated in favor of "
-            "domain_allowlist.",
-            RemovedInDjango41Warning,
-            stacklevel=2,
-        )
-        self.domain_allowlist = allowlist
-
-    def __init__(self, message=None, code=None, allowlist=None, *, whitelist=None):
-        if whitelist is not None:
-            allowlist = whitelist
-            warnings.warn(
-                "The whitelist argument is deprecated in favor of allowlist.",
-                RemovedInDjango41Warning,
-                stacklevel=2,
-            )
+    def __init__(self, message=None, code=None, allowlist=None):
         if message is not None:
             self.message = message
         if code is not None:
@@ -431,6 +402,15 @@ class MinValueValidator(BaseValidator):
         return a < b
 
 
+@deconstructible
+class StepValueValidator(BaseValidator):
+    message = _("Ensure this value is a multiple of step size %(limit_value)s.")
+    code = "step_size"
+
+    def compare(self, a, b):
+        return not math.isclose(math.remainder(a, b), 0, abs_tol=1e-9)
+
+
 @deconstructible
 class MinLengthValidator(BaseValidator):
     message = ngettext_lazy(
