@@ -6,6 +6,7 @@ from django.utils.functional import cached_property
 
 
 class DatabaseFeatures(BaseDatabaseFeatures):
+    minimum_database_version = (11,)
     allows_group_by_selected_pks = True
     can_return_columns_from_insert = True
     can_return_rows_from_bulk_insert = True
@@ -28,7 +29,6 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_combined_alters = True
     nulls_order_largest = True
     closed_cursor_error_class = InterfaceError
-    has_case_insensitive_like = False
     greatest_least_ignores_nulls = True
     can_clone_databases = True
     supports_temporal_subtraction = True
@@ -57,6 +57,10 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_deferrable_unique_constraints = True
     has_json_operators = True
     json_key_contains_list_matching_requires_list = True
+    supports_update_conflicts = True
+    supports_update_conflicts_with_target = True
+    supports_covering_indexes = True
+    can_rename_index = True
     test_collations = {
         "non_default": "sv-x-icu",
         "swedish_ci": "sv-x-icu",
@@ -79,10 +83,6 @@ class DatabaseFeatures(BaseDatabaseFeatures):
             "PositiveSmallIntegerField": "SmallIntegerField",
         }
 
-    @cached_property
-    def is_postgresql_11(self):
-        return self.connection.pg_version >= 110000
-
     @cached_property
     def is_postgresql_12(self):
         return self.connection.pg_version >= 120000
@@ -91,9 +91,13 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     def is_postgresql_13(self):
         return self.connection.pg_version >= 130000
 
-    has_websearch_to_tsquery = property(operator.attrgetter("is_postgresql_11"))
-    supports_covering_indexes = property(operator.attrgetter("is_postgresql_11"))
+    @cached_property
+    def is_postgresql_14(self):
+        return self.connection.pg_version >= 140000
+
+    has_bit_xor = property(operator.attrgetter("is_postgresql_14"))
     supports_covering_gist_indexes = property(operator.attrgetter("is_postgresql_12"))
+    supports_covering_spgist_indexes = property(operator.attrgetter("is_postgresql_14"))
     supports_non_deterministic_collations = property(
         operator.attrgetter("is_postgresql_12")
     )
