@@ -267,8 +267,9 @@ class RelatedField(FieldCacheMixin, Field):
             if not rel_is_hidden and clash_field.name == rel_name:
                 errors.append(
                     checks.Error(
-                        "Reverse accessor for '%s' clashes with field name '%s'."
-                        % (field_name, clash_name),
+                        f"Reverse accessor '{rel_opts.object_name}.{rel_name}' "
+                        f"for '{field_name}' clashes with field name "
+                        f"'{clash_name}'.",
                         hint=(
                             "Rename field '%s', or add/change a related_name "
                             "argument to the definition for field '%s'."
@@ -307,8 +308,9 @@ class RelatedField(FieldCacheMixin, Field):
             if not rel_is_hidden and clash_field.get_accessor_name() == rel_name:
                 errors.append(
                     checks.Error(
-                        "Reverse accessor for '%s' clashes with reverse accessor for "
-                        "'%s'." % (field_name, clash_name),
+                        f"Reverse accessor '{rel_opts.object_name}.{rel_name}' "
+                        f"for '{field_name}' clashes with reverse accessor for "
+                        f"'{clash_name}'.",
                         hint=(
                             "Add or change a related_name argument "
                             "to the definition for '%s' or '%s'."
@@ -342,7 +344,6 @@ class RelatedField(FieldCacheMixin, Field):
         return None
 
     def contribute_to_class(self, cls, name, private_only=False, **kwargs):
-
         super().contribute_to_class(cls, name, private_only=private_only, **kwargs)
 
         self.opts = cls._meta
@@ -496,7 +497,7 @@ class RelatedField(FieldCacheMixin, Field):
         When filtering against this relation, return the field on the remote
         model against which the filtering should happen.
         """
-        target_fields = self.get_path_info()[-1].target_fields
+        target_fields = self.path_infos[-1].target_fields
         if len(target_fields) > 1:
             raise exceptions.FieldError(
                 "The relation has multiple target fields, but only single target field "
@@ -538,7 +539,6 @@ class ForeignObject(RelatedField):
         swappable=True,
         **kwargs,
     ):
-
         if rel is None:
             rel = self.rel_class(
                 self,
@@ -562,6 +562,13 @@ class ForeignObject(RelatedField):
         self.to_fields = to_fields
         self.swappable = swappable
 
+    def __copy__(self):
+        obj = super().__copy__()
+        # Remove any cached PathInfo values.
+        obj.__dict__.pop("path_infos", None)
+        obj.__dict__.pop("reverse_path_infos", None)
+        return obj
+
     def check(self, **kwargs):
         return [
             *super().check(**kwargs),
@@ -822,6 +829,10 @@ class ForeignObject(RelatedField):
             )
         ]
 
+    @cached_property
+    def path_infos(self):
+        return self.get_path_info()
+
     def get_reverse_path_info(self, filtered_relation=None):
         """Get path from the related model to this field's model."""
         opts = self.model._meta
@@ -838,6 +849,10 @@ class ForeignObject(RelatedField):
             )
         ]
 
+    @cached_property
+    def reverse_path_infos(self):
+        return self.get_reverse_path_info()
+
     @classmethod
     @functools.lru_cache(maxsize=None)
     def get_lookups(cls):
@@ -961,6 +976,9 @@ class ForeignKey(ForeignObject):
         )
         self.db_constraint = db_constraint
 
+    def __class_getitem__(cls, *args, **kwargs):
+        return cls
+
     def check(self, **kwargs):
         return [
             *super().check(**kwargs),
@@ -1154,13 +1172,18 @@ class ForeignKey(ForeignObject):
         )
 
     def db_check(self, connection):
-        return []
+        return None
 
     def db_type(self, connection):
         return self.target_field.rel_db_type(connection=connection)
 
     def db_parameters(self, connection):
-        return {"type": self.db_type(connection), "check": self.db_check(connection)}
+        target_db_parameters = self.target_field.db_parameters(connection)
+        return {
+            "type": self.db_type(connection),
+            "check": self.db_check(connection),
+            "collation": target_db_parameters.get("collation"),
+        }
 
     def convert_empty_strings(self, value, expression, connection):
         if (not value) and isinstance(value, str):
@@ -1598,7 +1621,6 @@ class ManyToManyField(RelatedField):
                     (source_field_name, source),
                     (target_field_name, target),
                 ):
-
                     possible_field_names = []
                     for f in through._meta.fields:
                         if (
@@ -1750,12 +1772,17 @@ class ManyToManyField(RelatedField):
         linkfield1 = int_model._meta.get_field(self.m2m_field_name())
         linkfield2 = int_model._meta.get_field(self.m2m_reverse_field_name())
         if direct:
-            join1infos = linkfield1.get_reverse_path_info()
-            join2infos = linkfield2.get_path_info(filtered_relation)
+            join1infos = linkfield1.reverse_path_infos
+            if filtered_relation:
+                join2infos = linkfield2.get_path_info(filtered_relation)
+            else:
+                join2infos = linkfield2.path_infos
         else:
-            join1infos = linkfield2.get_reverse_path_info()
-            join2infos = linkfield1.get_path_info(filtered_relation)
-
+            join1infos = linkfield2.reverse_path_infos
+            if filtered_relation:
+                join2infos = linkfield1.get_path_info(filtered_relation)
+            else:
+                join2infos = linkfield1.path_infos
         # Get join infos between the last model of join 1 and the first model
         # of join 2. Assume the only reason these may differ is due to model
         # inheritance.
@@ -1773,9 +1800,17 @@ class ManyToManyField(RelatedField):
     def get_path_info(self, filtered_relation=None):
         return self._get_path_info(direct=True, filtered_relation=filtered_relation)
 
+    @cached_property
+    def path_infos(self):
+        return self.get_path_info()
+
     def get_reverse_path_info(self, filtered_relation=None):
         return self._get_path_info(direct=False, filtered_relation=filtered_relation)
 
+    @cached_property
+    def reverse_path_infos(self):
+        return self.get_reverse_path_info()
+
     def _get_m2m_db_table(self, opts):
         """
         Function that can be curried to provide the m2m table name for this
