@@ -1,8 +1,7 @@
-from urllib.parse import urlsplit, urlunsplit
+from urllib.parse import unquote, urlsplit, urlunsplit
 
 from asgiref.local import Local
 
-from django.utils.encoding import iri_to_uri
 from django.utils.functional import lazy
 from django.utils.translation import override
 
@@ -37,16 +36,16 @@ def reverse(viewname, urlconf=None, args=None, kwargs=None, current_app=None):
     if not isinstance(viewname, str):
         view = viewname
     else:
-        *path, view = viewname.split(':')
+        *path, view = viewname.split(":")
 
         if current_app:
-            current_path = current_app.split(':')
+            current_path = current_app.split(":")
             current_path.reverse()
         else:
             current_path = None
 
         resolved_path = []
-        ns_pattern = ''
+        ns_pattern = ""
         ns_converters = {}
         for ns in path:
             current_ns = current_path.pop() if current_path else None
@@ -76,15 +75,17 @@ def reverse(viewname, urlconf=None, args=None, kwargs=None, current_app=None):
             except KeyError as key:
                 if resolved_path:
                     raise NoReverseMatch(
-                        "%s is not a registered namespace inside '%s'" %
-                        (key, ':'.join(resolved_path))
+                        "%s is not a registered namespace inside '%s'"
+                        % (key, ":".join(resolved_path))
                     )
                 else:
                     raise NoReverseMatch("%s is not a registered namespace" % key)
         if ns_pattern:
-            resolver = get_ns_resolver(ns_pattern, resolver, tuple(ns_converters.items()))
+            resolver = get_ns_resolver(
+                ns_pattern, resolver, tuple(ns_converters.items())
+            )
 
-    return iri_to_uri(resolver._reverse_with_prefix(view, prefix, *args, **kwargs))
+    return resolver._reverse_with_prefix(view, prefix, *args, **kwargs)
 
 
 reverse_lazy = lazy(reverse, str)
@@ -100,8 +101,8 @@ def set_script_prefix(prefix):
     """
     Set the script prefix for the current thread.
     """
-    if not prefix.endswith('/'):
-        prefix += '/'
+    if not prefix.endswith("/"):
+        prefix += "/"
     _prefixes.value = prefix
 
 
@@ -111,7 +112,7 @@ def get_script_prefix():
     wishes to construct their own URLs manually (although accessing the request
     instance is normally going to be a lot cleaner).
     """
-    return getattr(_prefixes, "value", '/')
+    return getattr(_prefixes, "value", "/")
 
 
 def clear_script_prefix():
@@ -146,13 +147,12 @@ def get_urlconf(default=None):
 
 def is_valid_path(path, urlconf=None):
     """
-    Return True if the given path resolves against the default URL resolver,
-    False otherwise. This is a convenience method to make working with "is
-    this a match?" cases easier, avoiding try...except blocks.
+    Return the ResolverMatch if the given path resolves against the default URL
+    resolver, False otherwise. This is a convenience method to make working
+    with "is this a match?" cases easier, avoiding try...except blocks.
     """
     try:
-        resolve(path, urlconf)
-        return True
+        return resolve(path, urlconf)
     except Resolver404:
         return False
 
@@ -165,16 +165,23 @@ def translate_url(url, lang_code):
     """
     parsed = urlsplit(url)
     try:
-        match = resolve(parsed.path)
+        # URL may be encoded.
+        match = resolve(unquote(parsed.path))
     except Resolver404:
         pass
     else:
-        to_be_reversed = "%s:%s" % (match.namespace, match.url_name) if match.namespace else match.url_name
+        to_be_reversed = (
+            "%s:%s" % (match.namespace, match.url_name)
+            if match.namespace
+            else match.url_name
+        )
         with override(lang_code):
             try:
                 url = reverse(to_be_reversed, args=match.args, kwargs=match.kwargs)
             except NoReverseMatch:
                 pass
             else:
-                url = urlunsplit((parsed.scheme, parsed.netloc, url, parsed.query, parsed.fragment))
+                url = urlunsplit(
+                    (parsed.scheme, parsed.netloc, url, parsed.query, parsed.fragment)
+                )
     return url
