@@ -64,7 +64,7 @@ def technical_500_response(request, exc_type, exc_value, tb, status_code=500):
     reporter = get_exception_reporter_class(request)(request, exc_type, exc_value, tb)
     if request.accepts("text/html"):
         html = reporter.get_traceback_html()
-        return HttpResponse(html, status=status_code, content_type="text/html")
+        return HttpResponse(html, status=status_code)
     else:
         text = reporter.get_traceback_text()
         return HttpResponse(
@@ -72,7 +72,7 @@ def technical_500_response(request, exc_type, exc_value, tb, status_code=500):
         )
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def get_default_exception_reporter_filter():
     # Instantiate the default filter for the first time and cache it.
     return import_string(settings.DEFAULT_EXCEPTION_REPORTER_FILTER)()
@@ -92,6 +92,16 @@ def get_exception_reporter_class(request):
     )
 
 
+def get_caller(request):
+    resolver_match = request.resolver_match
+    if resolver_match is None:
+        try:
+            resolver_match = resolve(request.path)
+        except Http404:
+            pass
+    return "" if resolver_match is None else resolver_match._func_path
+
+
 class SafeExceptionReporterFilter:
     """
     Use annotations made by the sensitive_post_parameters and
@@ -368,6 +378,7 @@ class ExceptionReporter:
             c["request_FILES_items"] = self.request.FILES.items()
             c["request_COOKIES_items"] = self.request.COOKIES.items()
             c["request_insecure_uri"] = self._get_raw_insecure_uri()
+            c["raising_view_name"] = get_caller(self.request)
 
         # Check whether exception info is available
         if self.exc_type:
@@ -570,26 +581,6 @@ def technical_404_response(request, exception):
     if isinstance(urlconf, types.ModuleType):
         urlconf = urlconf.__name__
 
-    caller = ""
-    try:
-        resolver_match = resolve(request.path)
-    except Http404:
-        pass
-    else:
-        obj = resolver_match.func
-
-        if hasattr(obj, "view_class"):
-            obj = obj.view_class
-
-        if hasattr(obj, "__name__"):
-            caller = obj.__name__
-        elif hasattr(obj, "__class__") and hasattr(obj.__class__, "__name__"):
-            caller = obj.__class__.__name__
-
-        if hasattr(obj, "__module__"):
-            module = obj.__module__
-            caller = "%s.%s" % (module, caller)
-
     with builtin_template_path("technical_404.html").open(encoding="utf-8") as fh:
         t = DEBUG_ENGINE.from_string(fh.read())
     reporter_filter = get_default_exception_reporter_filter()
@@ -603,10 +594,10 @@ def technical_404_response(request, exception):
             "reason": str(exception),
             "request": request,
             "settings": reporter_filter.get_safe_settings(),
-            "raising_view_name": caller,
+            "raising_view_name": get_caller(request),
         }
     )
-    return HttpResponseNotFound(t.render(c), content_type="text/html")
+    return HttpResponseNotFound(t.render(c))
 
 
 def default_urlconf(request):
@@ -619,4 +610,4 @@ def default_urlconf(request):
         }
     )
 
-    return HttpResponse(t.render(c), content_type="text/html")
+    return HttpResponse(t.render(c))
